import java.io.*;
import java.net.URL;
import java.util.*;

/**
 * Une classe pour lire des informations d'un fichier des accs au serveur Web.
 * Actuellement, le fichier journal est cens simplement contenir
 * des informations de date et d'heure au format :
 *
 *    anne mois jour heure minute
 * Les entres du journal sont tries par ordre croissant de date.
 * 
 * @author David J. Barnes et Michael Kolling.
 * @version 2002.01.01
 */
public class LogfileReader
{
    // Le format des donnes du fichier journal.
    private String format;
    // L'endroit o est stock le contenu du journal au format
    // LogEntry.
    private ArrayList entries;
    // Un itrateur sur les entres.
    private Iterator dataIterator;
    
    /**
     * Cre un LogfileReader pour fournir des donnes  partir d'un fichier par dfaut.
     */
    public LogfileReader()
    {
        this("weblog.txt");
    }
    
    /**
     * Cre un LogfileReader qui fournira les donnes
     *  partir d'un fichier journal particulier. 
     * @param filename Le fichier des donnes du journal.
     */
    public LogfileReader(String filename)
    {
        // Le format des donnes.
        format = "Year Month(1-12) Day Hour Minute";       
        // Situe le fichier relativement  l'environnement actuel.
        URL fileURL = getClass().getClassLoader().getResource(filename);
        // O stocker les donnes.
        entries = new ArrayList();
        
        // Tentative de lire la totalit du jeu de donnes  partir du fichier.
        try{
            BufferedReader logfile =
                new BufferedReader(new FileReader(fileURL.getFile()));
            // Lit les lignes de donnes jusqu' la fin du fichier.
            String logline;
            logline = logfile.readLine();
            while(logline != null) {
                // Dcompose la ligne.
                LogEntry entry = new LogEntry(logline);
                entries.add(entry);
                logline = logfile.readLine();
            }
            logfile.close();
        }
        catch(IOException e) {
            System.out.println("Echec de lecture du fichier de donnes : " +
                               filename);
            System.out.println("Utilisation de donnes simules.");
            createSimulatedData(entries);
        }
        // Trie les entres par ordre croissant.
        Collections.sort(entries);
        reset();
    }
    
    /**
     * Le lecteur a-t-il d'autres donnes  fournir ?
     * @return true s'il y a d'autres donnes disponibles,
     *         false sinon.
     */
    public boolean hasMoreEntries()
    {
        return dataIterator.hasNext();
    }
    
    /**
     * Analyse la prochaine ligne du fichier journal et
     * la met  disposition de l'objet LogEntry.
     * 
     * @return Un LogEntry contenant les donnes de
     *         la prochaine ligne de journal.
     */
    public LogEntry nextEntry()
    {
        // Cre un LogEntry du contenu de la prochaine
        // ligne de journal.
        return (LogEntry) dataIterator.next();
    }
    
    /**
     * @return Une chane expliquant le format des donnes
     *         dans le fichier journal.
     */
    public String getFormat()
    {
        return format;
    }
    
    /**
     * Configure un nouvel itrateur pour fournir l'accs aux donnes.
     * Ceci permet  un seul fichier de donnes  traiter
     * plusieurs fois.
     */
    public void reset()
    {
        dataIterator = entries.iterator();
    }
    
    public void printData()
    {
        Iterator it = entries.iterator();
        while(it.hasNext()) {
            System.out.println((LogEntry) it.next());
        }
    }

    /**
     * Fournit un chantillon des donnes simules.
     * NB : pour simplifier la cration de ces donnes, aucun
     * jour aprs le 28 du mois ne sera gnr.
     * @param data O stocker les objets LogEntry simuls.
     */
    private void createSimulatedData(ArrayList data)
    {
        // Pour chaque donne (anne, mois, jour, heure, min) la valeur
        // valide la plus basse est indique.
        int[] lowest = { 2001, 1, 1, 0, 0, };
        // Pour chaque donne (anne, mois, jour, heure, min) la plage de
        // valeurs valides est indique (notez la simplification qui consiste
        //  avoir uniquement 28 jours dans le mois pour viter de gnrer des dates non-valides).
        int[] range = { 3, 12, 28, 24, 60 };
        // Utilise un jeu fixe pour gnrer les donnes alatoires,
        // afin que les donnes soient reproductibles.
        Random rand = new Random(12345);
        // Construit chaque ligne simule dans un tampon de chane.
        StringBuffer line = new StringBuffer();
        // Le nombre de lignes simules souhait.
        int numLines = 100;
        // Le nombre de valeurs de donnes par ligne simule.
        int itemsPerLine = lowest.length;
        for(int i = 0; i < numLines; i++) {
            for(int j = 0; j < itemsPerLine; j++) {
                int value = lowest[j]+rand.nextInt(range[j]);
                line.append(value);
                line.append(' ');
            }
            // Transforme la ligne en LogEntry.
            LogEntry entry = new LogEntry(line.toString());
            data.add(entry);
            line.setLength(0);
        }
    }
}
