import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;

/**
 * Classe Canvas - une classe qui permet un dessin graphique 
 * simple sur un fond.
 * 
 * @author Michael Kolling (mik)
 * @author Bruce Quig
 *
 * @version 1.8  (23.01.2002)
 */

public class Canvas
{
    private JFrame frame;
    private CanvasPane canvas;
    private Graphics2D graphic;
    private Color backgroundColor;
    private Image canvasImage;

    /**
     * Cre un Canvas avec une hauteur, une largeur et une couleur par dfaut de l'arrire-plan 
     * (300, 300, blanc).
     * @param title  titre qui doit apparatre dans le cadre Canvas     
     */
    public Canvas(String title)
    {
        this(title, 300, 300, Color.white);        
    }

    /**
     * Cre un Canvas avec une couleur par dfaut pour l'arrire-plan (blanc).
     * @param title  titre qui doit apparatre dans le cadre Canvas
     * @param width  largeur souhaite du fond
     * @param height  hauteur souhaite pour le fond
     */
    public Canvas(String title, int width, int height)
    {
        this(title, width, height, Color.white);
    }

    /**
     * Cre un Canvas.
     * @param title  titre qui doit apparatre dans le cadre Canvas
     * @param width  largeur souhaite du fond
     * @param height  hauteur souhaite pour le fond
     * @param bgClour  couleur d'arrire-plan souhaite pour le fond
     */
    public Canvas(String title, int width, int height, Color bgColor)
    {
        frame = new JFrame();
        canvas = new CanvasPane();
        frame.setContentPane(canvas);
        frame.setTitle(title);
        canvas.setPreferredSize(new Dimension(width, height));
        backgroundColor = bgColor;
        frame.pack();
    }

    /**
     * Dfinit la visibilit du fond et amne le fond au premier plan de l'cran
     * lorsqu'il est visible. Cette mthode peut aussi tre utilise pour amener un fond
     * dj visible  l'avant des autres fentres.
     * @param visible  valeur boolenne reprsentant la visibilit souhaite du
     * fond (true ou false) 
     */
    public void setVisible(boolean visible)
    {
        if(graphic == null) {
            // premire fois : instancie l'image hors cran et la remplit 
            // de la couleur de l'arrire-plan
            Dimension size = canvas.getSize();
            canvasImage = canvas.createImage(size.width, size.height);
            graphic = (Graphics2D)canvasImage.getGraphics();
            graphic.setColor(backgroundColor);
            graphic.fillRect(0, 0, size.width, size.height);
            graphic.setColor(Color.black);
        }
        frame.show();
    }

    /**
     * Fournit des informations sur la visibilit du Canvas.
     * @return  true si le fond est visible, false autrement
     */
    public boolean isVisible()
    {
        return frame.isVisible();
    }

    /**
     * Dessine le contour d'une forme donne sur le fond.
     * @param  shape  l'objet forme  dessiner sur le fond
     */
    public void draw(Shape shape)
    {
        graphic.draw(shape);
        canvas.repaint();
    }
 
    /**
     * Remplit les dimensions internes d'une forme donne avec la couleur 
     * donne du premier plan sur le fond.
     * @param  shape  l'objet forme  remplir 
     */
    public void fill(Shape shape)
    {
        graphic.fill(shape);
        canvas.repaint();
    }

    /**
     * Remplit les dimensions internes du cercle donn avec la couleur
     * donne du premier plan du fond.
     */
    public void fillCircle(int xPos, int yPos, int diameter)
    {
        Ellipse2D.Double circle = new Ellipse2D.Double(xPos, yPos, diameter, diameter);
        fill(circle);
    }

    /**
     * Remplit les dimensions internes du rectangle donn avec la couleur actuelle
     * du fond. C'est une mthode pratique. Un effet similaire 
     * peut tre obtenu avec la mthode fill.
     */
    public void fillRectangle(int xPos, int yPos, int width, int height)
    {
        fill(new Rectangle(xPos, yPos, width, height));
    }

    /**
     * Efface la totalit du fond.
     */
    public void erase()
    {
        Color original = graphic.getColor();
        graphic.setColor(backgroundColor);
        Dimension size = canvas.getSize();
        graphic.fill(new Rectangle(0, 0, size.width, size.height));
        graphic.setColor(original);
        canvas.repaint();
    }

    /**
     * Efface les dimensions internes du cercle donn. Il s'agit d'une
     * mthode pratique. Un effet similaire peut tre obtenu avec
     * la mthode erase.
     */
    public void eraseCircle(int xPos, int yPos, int diameter)
    {
        Ellipse2D.Double circle = new Ellipse2D.Double(xPos, yPos, diameter, diameter);
        erase(circle);
    }

    /**
     * Efface les dimensions internes du rectangle donn. Il s'agit d'une
     * mthode pratique. Un effet similaire peut tre obtenu avec
     * la mthode erase.
     */
    public void eraseRectangle(int xPos, int yPos, int width, int height)
    {
        erase(new Rectangle(xPos, yPos, width, height));
    }

    /**
     * Efface l'intrieur d'une forme donne  l'cran.
     * @param  shape  l'objet forme  effacer
     */
    public void erase(Shape shape)
    {
        Color original = graphic.getColor();
        graphic.setColor(backgroundColor);
        graphic.fill(shape);              // efface en remplissant avec la couleur d'arrire-plan
        graphic.setColor(original);
        canvas.repaint();
    }

    /**
     * Efface le contour d'une forme donne  l'cran.
     * @param  shape  l'objet forme  effacer
     */
    public void eraseOutline(Shape shape)
    {
        Color original = graphic.getColor();
        graphic.setColor(backgroundColor);
        graphic.draw(shape);  // efface en dessinant la couleur d'arrire-plan
        graphic.setColor(original);
        canvas.repaint();
    }

    /**
     * Dessine une image sur le fond.
     * @param  image   l'objet Image  afficher
     * @param  x       coordonne x pour le placement de l'Image 
     * @param  y       coordonne y pour le placement de l'Image 
     * @return  renvoie une valeur boolenne indiquant si l'image a t  
     *          compltement charge
     */
    public boolean drawImage(Image image, int x, int y)
    {
        boolean result = graphic.drawImage(image, x, y, null);
        canvas.repaint();
        return result;
    }

    /**
     * Dessine un String sur le fond.
     * @param  text   le String  afficher
     * @param  x      coordonne x pour le placement du texte
     * @param  y      coordonne y pour le placement du texte
     */
    public void drawString(String text, int x, int y)
    {
        graphic.drawString(text, x, y);   
        canvas.repaint();
    }

    /**
     * Efface un String du fond.
     * @param  text     le String  afficher
     * @param  x        coordonne x pour le placement du texte
     * @param  y        coordonne y pour le placement du texte
     */
    public void eraseString(String text, int x, int y)
    {
        Color original = graphic.getColor();
        graphic.setColor(backgroundColor);
        graphic.drawString(text, x, y);   
        graphic.setColor(original);
        canvas.repaint();
    }

    /**
     * Dessine un trait sur le fond.
     * @param  x1   coordonne x du dbut de ligne 
     * @param  y1   coordonne y du dbut de ligne
     * @param  x2   coordonne x de fin de ligne 
     * @param  y2   coordonne y de fin de ligne 
     */
    public void drawLine(int x1, int y1, int x2, int y2)
    {
        graphic.drawLine(x1, y1, x2, y2);   
        canvas.repaint();
    }

    /**
     * Dfinit la couleur de premier plan du fond.
     * @param  newColor   la nouvelle couleur du premier plan du fond 
     */
    public void setForegroundColor(Color newColor)
    {
        graphic.setColor(newColor);
    }

    /**
     * Renvoie la couleur actuelle du premier plan.
     * @return   la couleur du premier plan du fond 
     */
    public Color getForegroundColor()
    {
        return graphic.getColor();
    }

    /**
     * Dfinit la couleur d'arrire-plan du fond.
     * @param  newColor   la nouvelle couleur d'arrire-plan du fond
     */
    public void setBackgroundColor(Color newColor)
    {
        backgroundColor = newColor;   
        graphic.setBackground(newColor);
    }

    /**
     * Renvoie la couleur actuelle du fond
     * @return   la couleur d'arrire-pland du fond
     */
    public Color getBackgroundColor()
    {
        return backgroundColor;
    }

    /**
     * change la police actuelle utilise sur le fond
     * @param  newFont   nouvelle police  utiliser pour la sortie de String 
     */
    public void setFont(Font newFont)
    {
        graphic.setFont(newFont);
    }

    /**
     * Renvoie la police actuelle du fond.
     * @return     la police actuellement utilise
     **/
    public Font getFont()
    {
        return graphic.getFont();
    }

    /**
     * Dfinit la taille du fond.
     * @param  width    nouvelle largeur
     * @param  height   nouvelle hauteur 
     */
    public void setSize(int width, int height)
    {
        canvas.setPreferredSize(new Dimension(width, height));
        Image oldImage = canvasImage;
        canvasImage = canvas.createImage(width, height);
        graphic = (Graphics2D)canvasImage.getGraphics();
        graphic.drawImage(oldImage, 0, 0, null);
        frame.pack();
    }

    /**
     * Renvoie la taille du fond.
     * @return     Les dimensions actuelles du fond
     */
    public Dimension getSize()
    {
        return canvas.getSize();
    }

    /**
     * Attend un nombre spcifi de millisecondes avant de se terminer.
     * Ceci fournit une manire simple de spcifier un petit dlai qui peut tre
     * utilis pour produire des animations.
     * @param  milliseconds  le nombre
     */
    public void wait(int milliseconds)
    {
        try
        {
            Thread.sleep(milliseconds);
        } 
        catch (InterruptedException e)
        {
            // ignore l'exception du moment
        }
    }

    /************************************************************************
     * Classe CanvasPane imbrique - le composant rel du fond contenu dans 
     * le cadre Canvas. Il s'agit essentiellement d'un JPanel avec une capacit accrue 
     * pour actualiser l'image qui y est dessine.
     */
    private class CanvasPane extends JPanel
    {
        public void paint(Graphics g)
        {
            g.drawImage(canvasImage, 0, 0, null);
        }
    }
}
