import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.HashMap;

/**
 * Une vue graphique de la grille de simulation.
 * Chaque position est reprsente par un rectangle color 
 * selon son contenu. La vue utilise une couleur de fond par dfaut.
 * On peut dfinir les couleurs pour chaque espce par un appel 
 * la mthode setColor.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2002-04-23
 */
public class SimulatorView extends JFrame
{
    // Couleur pour les positions vides.
    private static final Color EMPTY_COLOR = Color.white;

    // Couleur pour les objets sans couleur dfinie.
    private static final Color UNKNOWN_COLOR = Color.gray;

    private final String STEP_PREFIX = "Pas: ";
    private final String POPULATION_PREFIX = "Population: ";
    private JLabel stepLabel, population;
    private FieldView fieldView;
    
    // Un objet map pour stocker les couleurs des participants  la simulation
    private HashMap colors;
    // Un objet statistics qui calcule et mmorise des informations sur la simulation
    private FieldStats stats;

    /**
     * Cre une vue de largeur (width) et hauteur (height) donnes.
     */
    public SimulatorView(int height, int width)
    {
        stats = new FieldStats();
        colors = new HashMap();

        setTitle("Simulation renards et lapins");
        stepLabel = new JLabel(STEP_PREFIX, JLabel.CENTER);
        population = new JLabel(POPULATION_PREFIX, JLabel.CENTER);
        
        setLocation(100, 50);
        
        fieldView = new FieldView(height, width);

        Container contents = getContentPane();
        contents.add(stepLabel, BorderLayout.NORTH);
        contents.add(fieldView, BorderLayout.CENTER);
        contents.add(population, BorderLayout.SOUTH);
        pack();
        setVisible(true);
    }
    
    /**
     * Dfinit la couleur  utiliser pour une classe donne d'animaux.
     */
    public void setColor(Class animalClass, Color color)
    {
        colors.put(animalClass, color);
    }

    /**
     * Renvoie la couleur utilise par une classe donne d'animaux.
     */
    private Color getColor(Class animalClass)
    {
        Color col = (Color)colors.get(animalClass);
        if(col == null) {
            // pas de couleur dfinie pour cette classe
            return UNKNOWN_COLOR;
        }
        else {
            return col;
        }
    }

    /**
     * Montre l'tat actuel du terrain.
     * @param step Pas d'itration
     * @param stats Etat du terrain  reprsenter
     */
    public void showStatus(int step, Field field)
    {
        if(!isVisible())
            setVisible(true);

        stepLabel.setText(STEP_PREFIX + step);

        stats.reset();
        fieldView.preparePaint();
            
        for(int row = 0; row < field.getDepth(); row++) {
            for(int col = 0; col < field.getWidth(); col++) {
                Object animal = field.getObjectAt(row, col);
                if(animal != null) {
                    stats.incrementCount(animal.getClass());
                    fieldView.drawMark(col, row, getColor(animal.getClass()));
                }
                else {
                    fieldView.drawMark(col, row, EMPTY_COLOR);
                }
            }
        }
        stats.countFinished();

        population.setText(POPULATION_PREFIX + stats.getPopulationDetails(field));
        fieldView.repaint();
    }

    /**
     * Dtermine si la simulation doit continuer.
     * @return true Si plus d'une espce est vivante.
     */
    public boolean isViable(Field field)
    {
        return stats.isViable(field);
    }
    
    /**
     * Fournit une vue graphique d'un terrain rectangulaire. C'est une classe
     * imbrique (une classe dfinie dans une classe) qui
     * dfinit un composant spcifique pour l'interface utilisateur.
     * Ce composant affiche le terrain.
     * Il s'agit de code IUG assez avanc - vous pouvez l'ignorer
     * dans vos projets si vous le voulez.
     */
    private class FieldView extends JPanel
    {
        private final int GRID_VIEW_SCALING_FACTOR = 6;

        private int gridWidth, gridHeight;
        private int xScale, yScale;
        Dimension size;
        private Graphics g;
        private Image fieldImage;

        /**
         * Cre une nouvelle vue.
         */
        public FieldView(int height, int width)
        {
            gridHeight = height;
            gridWidth = width;
            size = new Dimension(0, 0);
        }

        /**
         * Indique au gestionnaire d'IUG la taille que nous souhaitons avoir.
         */
        public Dimension getPreferredSize()
        {
            return new Dimension(gridWidth * GRID_VIEW_SCALING_FACTOR,
                                 gridHeight * GRID_VIEW_SCALING_FACTOR);
        }
        
        /**
         * Prpare un nouvel affichage. Comme le composant a pu tre
         * redimensionn, recalcule le facteur d'chelle.
         */
        public void preparePaint()
        {
            if(! size.equals(getSize())) {  // if the size has changed...
                size = getSize();
                fieldImage = fieldView.createImage(size.width, size.height);
                g = fieldImage.getGraphics();

                xScale = size.width / gridWidth;
                if(xScale < 1) {
                    xScale = GRID_VIEW_SCALING_FACTOR;
                }
                yScale = size.height / gridHeight;
                if(yScale < 1) {
                    yScale = GRID_VIEW_SCALING_FACTOR;
                }
            }
        }
        
        /**
         * Colorie une position de la grille du terrain avec une couleur donne.
         */
        public void drawMark(int x, int y, Color color)
        {
            g.setColor(color);
            g.fillRect(x * xScale, y * yScale, xScale-1, yScale-1);
        }

        /**
         * Le composant de visualisation du terrain doit tre raffich. Copier
         * l'image interne sur l'cran.
         */
        public void paintComponent(Graphics g)
        {
            if(fieldImage != null) {
                g.drawImage(fieldImage, 0, 0, null);
            }
        }
    }
}
