import java.util.List;
import java.util.Iterator;
import java.util.Random;

/**
 * Un modle lmentaire de renard (fox).
 * Un renard vieillit, se dplace, mange des lapins (rabbits) et meurt.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2002-04-11
 */
public class Fox extends Animal
{
    // Caractristiques partages par tous les renards (foxes) (champs statistiques).
    
    // L'ge  partir duquel un renard peut se reproduire
    private static final int BREEDING_AGE = 10;
    // L'ge maximal d'un renard
    private static final int MAX_AGE = 150;
    // La probabilit de reproduction d'un renard
    private static final double BREEDING_PROBABILITY = 0.09;
    // La taille maximale d'une porte
    private static final int MAX_LITTER_SIZE = 3;
    // La valeur alimentaire d'un lapin. Correspond au nombre de pas de simulation
    // qu'un renard peut effectuer avant de devoir manger.
    private static final int RABBIT_FOOD_VALUE = 4;
    // Un gnrateur de nombres alatoires commun pour contrler les reproductions.
    private static final Random rand = new Random();
    
    // Caractristiques individuelles (champs d'instance).

    // La rserve alimentaire du renard, augmente lorsque le renard mange des lapins
    private int foodLevel;

    /**
     * Cre un renard. Un  renard peut tre cr comme nouveau n (ge nul et non affam)
     * ou avec un ge alatoire.
     * 
     * @param randomAge Si true, le renard aura un ge alatoire et un niveau de faim.
     */
    public Fox(boolean randomAge)
    {
        super();
        if(randomAge) {
            setAge(rand.nextInt(MAX_AGE));
            foodLevel = rand.nextInt(RABBIT_FOOD_VALUE);
        }
        else {
            // laisser l'ge  zro
            foodLevel = RABBIT_FOOD_VALUE;
        }
    }
    
    /**
     * Ce que fait le renard la plupart du temps: il chasse des lapins.
     * Durant cette chasse, il peut se reproduire, mourir de faim
     * ou mourir de vieillesse.
     */
    public void act(Field currentField, Field updatedField, List newAnimals)
    {
        incrementAge();
        incrementHunger();
        if(isAlive()) {
            // De nouveaux renards sont ns dans les positions adjacentes
            int births = breed();
            for(int b = 0; b < births; b++) {
                Fox newFox = new Fox(false);
                newAnimals.add(newFox);
                newFox.setLocation(updatedField.randomAdjacentLocation(getLocation()));
                updatedField.place(newFox);
            }
            // Se dplacer vers une source de nourriture si on en trouve une
            Location newLocation = findFood(currentField, getLocation());
            if(newLocation == null) {  // pas de nourriture trouve - se dplacer au hasard
                newLocation = updatedField.freeAdjacentLocation(getLocation());
            }
            if(newLocation != null) {
                setLocation(newLocation);
                updatedField.place(this);  // fixer la position
            }
            else {
                // dplacement impossible et position courante occupe
                setDead();
            }
        }
    }
    
    /**
     * Augmente l'ge. Peut provoquer la mort (de vieillesse).
     */
    private void incrementAge()
    {
        setAge(getAge() + 1);
        if(getAge() > MAX_AGE) {
            setDead();
        }
    }
    
    /**
     * Rend ce renard plus affam. Peut provoquer la mort (de faim).
     */
    private void incrementHunger()
    {
        foodLevel--;
        if(foodLevel <= 0) {
            setDead();
        }
    }
    
    /**
     * Indique au renard de chercher des lapins dans les positions adjacentes.
     * @param field Terrain o chercher
     * @param location Position actuelle du renard
     * @return La position o la nourriture a t trouve, null si rien trouv
     */
    private Location findFood(Field field, Location location)
    {
        Iterator adjacentLocations =
                          field.adjacentLocations(location);
        while(adjacentLocations.hasNext()) {
            Location where = (Location) adjacentLocations.next();
            Object animal = field.getObjectAt(where);
            if(animal instanceof Rabbit) {
                Rabbit rabbit = (Rabbit) animal;
                if(rabbit.isAlive()) { 
                    rabbit.setDead();
                    foodLevel = RABBIT_FOOD_VALUE;
                    return where;
                }
            }
        }
        return null;
    }
        
    /**
     * Gnre un nombre de naissances,
     * si le renard peut se reproduire
     * @return Le nombre de naissances (peut tre nul)
     */
    private int breed()
    {
        int births = 0;
        if(canBreed() && rand.nextDouble() <= BREEDING_PROBABILITY) {
            births = rand.nextInt(MAX_LITTER_SIZE) + 1;
        }
        return births;
    }
    
    /**
     * @return A string representation of the fox.
     */
    public String toString()
    {
        return "Fox, age " + getAge();
    }

    /**
     * Un renard peut se reproduire s'il a atteint l'ge adulte.
     */
    private boolean canBreed()
    {
        return getAge() >= BREEDING_AGE;
    }
}
