import java.util.Random;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collections;
import java.awt.Color;

 /**
 * Une simulation lmentaire proies-prdateurs, fonde sur un terrain
 * contenant des lapins (rabbits) et des renards (foxes).
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2002-04-09
 */
public class Simulator
{
    // Les variables private static final reprsentent 
    // les informations de configuration de la simulation.
    // Largeur par dfaut de la grille
    private static final int DEFAULT_WIDTH = 50;
    // Hauteur par dfaut de la grille
    private static final int DEFAULT_DEPTH = 50;
    // La probabilit qu'un renard soit cr dans une position de la grille
    private static final double FOX_CREATION_PROBABILITY = 0.02;
    // La probabilit qu'un lapin soit cr dans une position de la grille
    private static final double RABBIT_CREATION_PROBABILITY = 0.08;    

    // La liste des animaux du terrain
    private List animals;
    // La liste des nouveaux ns
    private List newAnimals;
    // L'tat courant du terrain
    private Field field;
    // Un deuxime terrain, utilis pour construire l'tat suivant de la simulation.
    private Field updatedField;
    // Le pas courant de la simulation.
    private int step;
    // Une reprsentation graphique de la simulation.
    private SimulatorView view;

    /**
     * Construit un terrain de simulation avec la taille par dfaut.
     */
    public Simulator()
    {
        this(DEFAULT_DEPTH, DEFAULT_WIDTH);
    }
    
    /**
     * Cre un terrain de simulation avec une taille donne.
     * @param depth Hauteur du terrain. Doit tre suprieure  zro.
     * @param width Largeur du terrain. Doit tre suprieure  zro.
     */
    public Simulator(int depth, int width)
    {
        if(width <= 0 || depth <= 0) {
            System.out.println("Les dimensions doivent tre suprieures  zro.");
            System.out.println("Valeurs par dfaut utilises.");
            depth = DEFAULT_DEPTH;
            width = DEFAULT_WIDTH;
        }
        animals = new ArrayList();
        newAnimals = new ArrayList();
        field = new Field(depth, width);
        updatedField = new Field(depth, width);

        // Crer une vue de l'tat de chaque position du terrain.
        view = new SimulatorView(depth, width);
        view.setColor(Fox.class, Color.blue);
        view.setColor(Rabbit.class, Color.orange);
        
        // Dfinir un point de dpart valide.
        reset();
    }
    
    /**
     * Excute la simulation  partir de son tat courant pour une priode assez longue,
     * par exemple 500 pas.
     */
    public void runLongSimulation()
    {
        simulate(50000);
    }
    
    /**
     * Excute la simulation  partir de son tat courant pour un nombre de pas donn.
     * Arrte avant si elle n'est plus viable.
     */
    public void simulate(int numSteps)
    {
        for(int step = 1; step <= numSteps && view.isViable(field); step++) {
            simulateOneStep();
        }
    }
    
    /**
     * Excute la simulation  partir de son tat courant pour un pas.
     * Itre sur tout le terrain en mettant  jour l'tat de chaque
     * renard et de chaque lapin.
     */
    public void simulateOneStep()
    {
        step++;
        newAnimals.clear();
        
        // faire agir les animaux.
        for(Iterator iter = animals.iterator(); iter.hasNext(); ) {
            Animal animal = (Animal)iter.next();
            if(animal.isAlive()) {
                animal.act(field, updatedField, newAnimals);
            }
            else {
                iter.remove();   // supprimer les animaux morts de la collection
            }
        }
        // ajouter les nouveaux ns  la liste des animaux
        animals.addAll(newAnimals);
        
        // Echanger le terrain (field) et sa version mise  jour (updatedField)  la fin d'un pas.
        Field temp = field;
        field = updatedField;
        updatedField = temp;
        updatedField.clear();

        // afficher le nouveau terrain  l'cran
        view.showStatus(step, field);
    }
        
    /**
     * Rinitialise la simulation.
     */
    public void reset()
    {
        step = 0;
        animals.clear();
        field.clear();
        updatedField.clear();
        populate(field);
        
        // Montrer l'tat de dpart.
        view.showStatus(step, field);
    }
    
    /**
     * Peuple le terrain avec des renards et des lapins.
     */
    private void populate(Field field)
    {
        Random rand = new Random();
        field.clear();
        for(int row = 0; row < field.getDepth(); row++) {
            for(int col = 0; col < field.getWidth(); col++) {
                if(rand.nextDouble() <= FOX_CREATION_PROBABILITY) {
                    Fox fox = new Fox(true);
                    fox.setLocation(row, col);
                    animals.add(fox);
                    field.place(fox);
                }
                else if(rand.nextDouble() <= RABBIT_CREATION_PROBABILITY) {
                    Rabbit rabbit = new Rabbit(true);
                    rabbit.setLocation(row, col);
                    animals.add(rabbit);
                    field.place(rabbit);
                }
                // else laisser la position vide.
            }
        }
        Collections.shuffle(animals);
    }
    
    public static void main(String[] args) {
    	Simulator sim = new Simulator();
    	sim.runLongSimulation();
    }
}
