import java.io.*;
import java.net.URL;

/**
 * Fournit un ensemble d'oprations sur fichiers pour un carnet d'adresses (AddressBook).
 * Ces mthodes illustrent une partie des caractristiques
 * du paquetage java.io.
 * 
 * @author David J. Barnes and Michael Kolling.
 * @version 2002.06.13
 */
public class AddressBookFileHandler
{
    // Le carnet d'adresses sur lequel on effectue les E/S.
    private AddressBook book;
    // Le nom de fichier utilis pour stocker les rsultats d'une recherche.
    private static final String RESULTS_FILE = "results.txt";

    /**
     * Constructeur des objets de la classe FileHandler.
     * @param book Le carnet d'adresses  utiliser.
     */
    public AddressBookFileHandler(AddressBook book)
    {
        this.book = book;
    }
    
    /**
     * Sauve les rsultats d'une recherche sur un carnet d'adresses dans
     * le fichier "results.txt", dans le rpertoire projet.
     * @param keyPrefix Le prfixe de recherche.
     */
    public void saveSearchResults(String keyPrefix) throws IOException
    {
        File resultsFile = makeAbsoluteFilename(RESULTS_FILE);
        ContactDetails[] results = book.search(keyPrefix);
        FileWriter writer = new FileWriter(resultsFile);
        for(int i = 0; i < results.length; i++) {
            writer.write(results[i].toString());
            writer.write('\n');
            writer.write('\n');
        }
        writer.close();
    }
    
    /**
     * Affiche les rsultats de la dernire recherche enregistre par
     * un appel  saveSearchResults. La sortie ayant lieu sur la console,
     * tout problme est signal directement par la prsente mthode.
     */
    public void showSearchResults()
    {
        File resultsFile = makeAbsoluteFilename(RESULTS_FILE);
        BufferedReader reader = null;
        try {
            reader = new BufferedReader(
                        new FileReader(resultsFile));
            System.out.println("Rsultats ...");
            String line;
            line = reader.readLine();
            while(line != null) {
                System.out.println(line);
                line = reader.readLine();
            }
            System.out.println();
        }
        catch(FileNotFoundException e) {
            System.out.println("Impossible de trouver le fichier: " +
                               resultsFile);
        }
        catch(IOException e) {
            System.out.println("Erreur pendant la lecture du fichier: " +
                               resultsFile);
        }
        finally {
            if(reader != null) {
                // Capture toutes les exceptions, mais ne fait rien
                // de plus.
                try {
                    reader.close();
                }
                catch(IOException e) {
                    System.out.println("Erreur  la fermeture de: " +
                                       resultsFile);
                }
            }
        }
    }

    /**
     * Ajoute des entres au carnet d'adresses,  partir d'un fichier texte.
     * On suppose que le fichier contient un lment par ligne,
     * plus une ligne vide pour chaque entre:
     *     nom \n tlphone \n adresse \n \n
     * Une ligne peut tre vide si l'information correspondante est manquante.
     * @param filename Le fichier texte contenant les informations.
     * @throws IOException Sur une erreur de lecture.
     */
    public void addEntriesFromFile(String filename) throws IOException
    {
        // S'assurer que le fichier est trouve.
        URL resource = getClass().getResource(filename);
        if(resource == null) {
            throw new FileNotFoundException(filename);
        }
        filename = resource.getFile();
        BufferedReader reader = new BufferedReader(
                                   new FileReader(filename));
        String name;
        name = reader.readLine();
        while(name != null) {
            String phone = reader.readLine();
            String address = reader.readLine();
            // Discard the separating blank line.
            reader.readLine();
            book.addDetails(new ContactDetails(name, phone,
                                               address));
            name = reader.readLine();
        }
        reader.close();
    }
    
    /**
     * Lit la version binaire d'un carnet d'adresses  partir d'un fichier.
     * Si le nom de fichier n'est pas absolu, on suppose qu'il est
     * relatif au rpertoire courant du projet.
     * @param sourceFile The file from where the details are to be read.
     * @return L'objet carnet d'adresses.
     * @throws IOException Si le processus de lecture rencontre une erreur.
     */
    public AddressBook readFromFile(String sourceFile)
        throws IOException, ClassNotFoundException
    {
        // Make sure the file can be found.
        URL resource = getClass().getResource(sourceFile);
        if(resource == null) {
            throw new FileNotFoundException(sourceFile);
        }
        String source = resource.getFile();
        ObjectInputStream is = new ObjectInputStream(
                                new FileInputStream(source));
        AddressBook savedBook = (AddressBook) is.readObject();
        is.close();
        return savedBook;
    }
    
    /**
     * Sauvegarde une version binaire du carnet d'adresses dans le fichier donn.
     * Si le nom de fichier n'est pas absolu, on suppose qu'il est
     * relatif au rpertoire courant du projet.
     * @param destinationFile Le fichier o enregistrer les informations.
     * @throws IOException Si la sauvegarde rencontre une erreur.
     */
    public void saveToFile(String destinationFile) throws IOException
    {
        File destination = makeAbsoluteFilename(destinationFile);
        ObjectOutputStream os = new ObjectOutputStream(
                                    new FileOutputStream(destination));
        os.writeObject(book);
        os.close();
    }
    
    /**
     * Cre un fichier  partir du nom de fichier donn.
     * Si le nom de fichier est dj absolu, l'utiliser tel quel,
     * sinon supposer qu'il est relatif au rpertoire courant
     * du projet.
     */
    private File makeAbsoluteFilename(String filename)
    {
        File file = new File(filename);
        if(!file.isAbsolute()) {
            file = new File(getProjectFolder(), filename);
        }
        return file;
    }
    
    /**
     * Essaie de dterminer le nom du rpertoire courannt du projet.
     * Ceci implique de localiser le chemin du fichier '.class'
     * de cette classe, puis extraire le nom du rpertoire
     * qui le contient.
     * @return Le rpertoire courant du projet.
     */
    private File getProjectFolder()
    {
         String myClassFile = getClass().getName() + ".class";
         URL url = getClass().getResource(myClassFile);
         return new File(url.getPath()).getParentFile();
    }
}
